﻿  ////////////////////////////////////////////////////////////
// Classe CarMath pour écrire des expressions Mathématiques  //
// Développée pour TracenPoche (c) Sésamath E.OSTENNE        //
// contact@sesamath.net emmanuel.ostenne@sesamath.net        //  
// sous licence CeCILL 2 - 18/04/08                          //
///////////////////////////////////////////////////////////////
class CarMath {
	//
	// transforme les symboles suivant placés entre caractères marqueur (£ par défaut)
	// Exemples : £alpha£ affiche un caractère grec alpha
	//            £perp£ affiche un symbole perpendiculaire
	//            filtrage des fonctions de mises en formes :
	//                   angle, crochets, exposant, fraction, indice,paraenthèses,système,racine, vecteur :
	//                   laisse les £a,£c,£e,£f,£i,£p,£r,£s,£v (si le marqueur est £)
	//                   utilisé par TexteMath
	//
	private var le_marqueur:String;
	private var le_symbolesFont:Array;
	private var le_symbolesUnicode:Array;
	//
	private var const_symboleFont : Array = ["Alpha", "Beta", "Chi", "Delta", "Epsilon", "Phi", "Gamma", "Eta", "Iota", "Phi2", "Kappa", "Lambda", "Mu", "Nu", "Omicron", "Pi", "Theta", "Rho", "Sigma", "Tau", "Upsilon", "Sampi", "Omega", "Xi", "Psi", "Dzeta", "[", "3pts", "]", "perp", "souligne", "barre", "alpha", "beta", "chi", "delta", "epsilon", "phi", "gamma", "eta", "iota", "phi2", "kappa", "lambda", "mu", "nu", "omicron", "pi", "theta", "rho", "sigma", "tau", "upsilon", "sampi", "omega", "xi", "psi", "dzeta", "{", "|", "}", "tilde"];
	private var const_symbolesUnicode : Array = [["alpha2","\u03B1"],["plus","\u002B"],["moins","\u2212"],["fois", "\u00D7"],["divise", "\u00F7"],["petitf", "\u0192"], ["moins", "\u2010"], ["petitebarre", "\u2013"], ["grandebarre", "\u2014"], ["prime", "\u2019"], ["seconde", "\u201D"], ["puceronde", "\u2022"], ["grandC", "\u2102"], ["euler", "\u2102"], ["petitg", "\u210A"], ["petith", "\u210F"], ["Ironde", "\u2111"], ["Lronde", "\u2112"], ["Cronde", "\u2113"], ["grandN", "\u2115"], ["Pronde", "\u2118"], ["grandQ", "\u211A"], ["Rronde", "\u211C"], ["grandR", "\u211D"], ["grandZ", "\u2124"], ["Eronde", "\u2130"], ["Fronde", "\u2131"], ["Nronde", "\u2135"], ["flecheG", "\u2190"], ["flecheH", "\u2191"], ["flecheD", "\u2192"], ["flecheB", "\u2193"], ["flecheDG", "\u2194"], ["flecheGD", "\u2194"], ["flecheHB", "\u2195"], ["flecheBH", "\u2195"], ["croissant", "\u2197"], ["descroissant", "\u2198"], ["alaligneadroite", "\u21B3"], ["alaligneagauche", "\u21B5"], ["doubleflecheG", "\u21D0"], ["doubleflecheD", "\u21D2"], ["doubleflecheDG", "\u21D4"], ["doubleflecheGD", "\u21D4"], ["flecheGbarre", "\u21E4"], ["flecheDbarre", "\u21E5"], ["flecheGcreuse", "\u21E6"], ["flecheHcreuse", "\u21E7"], ["flecheDcreuse", "\u21E8"], ["flecheBcreuse", "\u21E9"], ["qqsoit", "\u2200"], ["pourtout", "\u2200"], ["quelquesoit", "\u2200"], ["complement", "\u2201"], ["differentielpartiel", "\u2202"], ["ilexiste", "\u2203"], ["ilnexistepas", "\u2204"], ["vide", "\u2205"], ["nabla", "\u2207"], ["appartienta", "\u2208"], ["nappartientpasa", "\u2209"], ["contient", "\u220B"], ["petitcontient", "\u220D"], ["grandproduit", "\u220F"], ["grandcoproduit", "\u2210"], ["grandesomme", "\u2211"], ["petitebarrefine", "\u2212"], ["moinsouplus", "\u2213"], ["antislash", "\u2216"], ["asterisque", "\u2217"], ["racine", "\u221A"], ["proportionnela", "\u221D"], ["infini", "\u221E"], ["angle", "\u2221"], ["anglespherique", "\u2222"], ["divise", "\u2223"], ["nedivisepas", "\u2224"], ["parallelea", "\u2225"], ["nestpasparallelea", "\u2226"], ["etlogique", "\u2227"], ["oulogique", "\u2228"], ["inter", "\u2229"], ["intersection", "\u2229"], ["union", "\u222A"], ["integrale", "\u222B"], ["doubleintegrale", "\u222C"], ["tripleintegrale", "\u222D"], ["integralecurviligne", "\u222E"], ["integralesurfacique", "\u222F"], ["integralevolumique", "\u2230"], ["egaleasymptotiquea", "\u2243"], ["environdroit", "\u2243"], ["environegala", "\u2245"], ["environ", "\u2248"], ["egalpardefinition", "\u225D"], ["differentde", "\u2260"], ["identiquea", "\u2261"], ["inferieura", "\u2264"], ["superieura", "\u2265"], ["inclusdans", "\u2282"], ["contient2", "\u2283"], ["nestpasinclusdans", "\u2282"], ["necontientpas", "\u2283"], ["sommedirecte", "\u2295"], ["differencedirecte", "\u2296"], ["produittensoriel", "\u2297"], ["divisiondirecte", "\u2298"], ["produitdirect", "\u2299"], ["top", "\u22A4"], ["perpendiculairea", "\u22A5"], ["antecedentde", "\u22B6"], ["imagede", "\u22B7"], ["angledroitarc", "\u22BE"], ["point", "\u22C5"], ["pv", ";"] ];
	//
	private var le_traiteSymboles:Function;
	//
	//
	private var la_chaine_originale:String;
	private var la_chaine_convertie:String;
	//
	public function CarMath(chaine:String, marqueur:String, symboleFont:Array, symbolesUnicode:Array) {
		if ((marqueur == undefined) || (marqueur == null)) {
			this.le_marqueur = "£";
		} else {
			this.le_marqueur = marqueur;
		}
		if ((symboleFont == undefined) || (symboleFont == null)) {
			this.le_symbolesFont = const_symboleFont;
		}
		if ((symbolesUnicode == undefined) || (symbolesUnicode == null)) {
			this.le_symbolesUnicode = const_symbolesUnicode;
		}
		//différence PC et MAC/LINUX
		var osCap : String = System.capabilities.os.toUpperCase();
		if (osCap.indexOf("WINDOWS") == 0) {
			//trace("symboles PC");
			this.le_traiteSymboles = this.traiteSymbolesPC;
		} else {
			if (osCap.indexOf("MAC") == 0) {
				//trace("symboles MAC/LINUX");
				this.le_traiteSymboles = this.traiteSymbolesMAC;
			}else {
				//trace("symboles MAC/LINUX");
				this.le_traiteSymboles = this.traiteSymbolesLINUX;
			}
		}
		if ((chaine == undefined) || (chaine == null)) {
			this.la_chaine_originale = "";
			this.la_chaine_convertie = "";
		} else {
			this.la_chaine_originale = chaine;
			this.la_chaine_convertie = this.le_traiteSymboles(this.la_chaine_originale);
		}
	}
	//
	public function get chaine() : String {
		//this.la_chaine_convertie = traiteSymboles(this.la_chaine_originale);
		return this.la_chaine_convertie;
	}
	//
	public function set chaine(c:String) {
		if (c != this.la_chaine_originale) {
			this.la_chaine_originale = c;
			this.la_chaine_convertie = this.le_traiteSymboles(this.la_chaine_originale);
		}
	}
	//
	public function get marqueur() : String {
		return this.le_marqueur;
	}
	//
	public function set marqueur(m:String) {
		if (m != this.le_marqueur) {
			this.le_marqueur = m;
		}
	}
	//
	private function estdansliste(c : String, liste : Array) : Boolean {
		for (var i : Number = 0; i<=liste.length; i++) {
			if (c == liste[i]) {
				return true;
			}
		}
		return false;
	}
	//
	private function traiteSymbolesPC(ch : String) : String {
		if (ch.indexOf(this.le_marqueur) == -1) {
			return ch;
		}
		//var nblivrefin = 0;
		var p1 : Array = ch.split(this.le_marqueur);
		//trace(p1);
		var resultat : String = "";
		var c : String = "";
		var m : Number = 0;
		var t : String = "";
		var k : Number =0;
		var r : Number =0;
		var symboleavant : Boolean = false;
		resultat = p1[0];
		for (var i : Number = 1; i<p1.length; i++) {
			//trace(p1[i]);
			c = p1[i];
			m = c.indexOf("(");
			if (m != 1) {
				for (k = 0; k<this.le_symbolesFont.length; k++) {
					if (c == this.le_symbolesFont[k]) {
						resultat = resultat+"<font face='symbol'>"+String.fromCharCode(65+k)+"</font>";
						k = 1000;
					}
				}
				for (r = 0; r<this.le_symbolesUnicode.length; r++) {
					if (c == this.le_symbolesUnicode[r][0]) {
						resultat = resultat+"<font face='Lucida Sans Unicode' name='Lucida Sans Unicode'>"+this.le_symbolesUnicode[r][1]+"</font>";
						r = 1000;
					}
				}
				if ((k == this.le_symbolesFont.length) and (r == this.le_symbolesUnicode.length)) {
					resultat = resultat+c;
					symboleavant = false;
				} else symboleavant = true;
			} else {
				t = c.substr(0, m);
				if (this.estdansliste(t, ["?","a","c","d","e","f","g","i","l","n","p","r","s","u","v"])) {
					if(symboleavant==false) {
						resultat = resultat+this.le_marqueur+c;
					} else {
						resultat = resultat+c;
					}
				} else {
					//resultat = resultat+"?"+c+"?";
					resultat = resultat+c;
				}
			}
		}
		//trace("->resultat:"+resultat);
		return resultat;
	}
	//
	private function traiteSymbolesLINUX(ch : String) : String {
		if (ch.indexOf(this.le_marqueur) == -1) {
			return ch;
		}
		//var nblivrefin = 0;
		var p1 : Array = ch.split(this.le_marqueur);
		//trace(p1);
		var resultat : String = "";
		var c : String = "";
		var m : Number = 0;
		var t : String = "";
		var k : Number =0;
		var r : Number =0;
		var symboleavant : Boolean = false;
		resultat = p1[0];
		for (var i : Number = 1; i<p1.length; i++) {
			//trace(p1[i]);
			c = p1[i];
			m = c.indexOf("(");
			if (m != 1) {
				for (k = 0; k<this.le_symbolesFont.length; k++) {
					if (c == this.le_symbolesFont[k]) {
						resultat = resultat+"<font face='symbol'>"+String.fromCharCode(65+k)+"</font>";
						k = 1000;
					}
				}
				for (r = 0; r<this.le_symbolesUnicode.length; r++) {
					if (c == this.le_symbolesUnicode[r][0]) {
						resultat = resultat+"<font face='Lucida Sans Unicode' name='Lucida Sans Unicode'>"+this.le_symbolesUnicode[r][1]+"</font>";
						r = 1000;
					}
				}
				if ((k == this.le_symbolesFont.length) and (r == this.le_symbolesUnicode.length)) {
					resultat = resultat+c;
					symboleavant = false;
				} else symboleavant = true;
			} else {
				t = c.substr(0, m);
				if (this.estdansliste(t, ["?","a","c","d","e","f","g","i","l","n","p","r","s","u","v"])) {
					if(symboleavant==false) {
						resultat = resultat+this.le_marqueur+c;
					} else {
						resultat = resultat+c;
					}
				} else {
					//resultat = resultat+"?"+c+"?";
					resultat = resultat+c;
				}
			}
		}
		//trace("->resultat:"+resultat);
		return resultat;
	}
	//
	private function traiteSymbolesMAC(ch : String) : String {
		if (ch.indexOf(this.le_marqueur) == -1) {
			return ch;
		}
		//var nblivrefin = 0;
		var p1 : Array = ch.split(this.le_marqueur);
		//trace(p1);
		var resultat : String = "";
		var c : String = "";
		var m : Number = 0;
		var t : String = "";
		var k : Number =0;
		var r : Number =0;
		var symboleavant : Boolean = false;
		resultat = p1[0];
		for (var i : Number = 1; i<p1.length; i++) {
			//trace(p1[i]);
			c = p1[i];
			m = c.indexOf("(");
			if (m != 1) {
				for (k = 0; k<this.le_symbolesFont.length; k++) {
					if (c == this.le_symbolesFont[k]) {
						resultat = resultat+"<font face='symbol'>"+String.fromCharCode(65+k)+"</font>";
						k = 1000;
					}
				}
				for (r = 0; r<this.le_symbolesUnicode.length; r++) {
					if (c == this.le_symbolesUnicode[r][0]) {
						resultat = resultat+this.le_symbolesUnicode[r][1];
						r = 1000;
					}
				}
				if ((k == this.le_symbolesFont.length) and (r == this.le_symbolesUnicode.length)) {
					resultat = resultat+c;
					symboleavant = false;
				} else symboleavant = true;
			} else {
				t = c.substr(0, m);
				if (this.estdansliste(t, ["?","a","c","d","e","f","g","i","l","n","p","r","s","u","v"])) {
					if(symboleavant==false) {
						resultat = resultat+this.le_marqueur+c;
					} else {
						resultat = resultat+c;
					}
				} else {
					//resultat = resultat+"?"+c+"?";
					resultat = resultat+c;
				}
			}
		}
		//trace("->resultat:"+resultat);
		return resultat;
	}
}
